///////////////////////////////////////////////////////////////////////////
//
//   Copyright 2010 Alberto González Palomo
//   Author: Alberto González Palomo - http://sentido-labs.com/
//
//   This file is part of Tutorial Toolkit, abbreviated as TTK.
//
//   TTK is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 3 of the License, or
//   (at your option) any later version.
//
//   TTK is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with TTK; if not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////////////

package com.sentidolabs.ttk;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.Vector;
import java.util.Iterator;

/**
 * Call-out with the appearance of a thought bubble.
 * The links to the targets of this call-out are drawn as chains of ellipses.
 */
public class ThoughtBubbleCallOut extends CallOut
{
    public ThoughtBubbleCallOut()
    {
        setStroke(new Color(10, 10, 10), 2.0f);
        setBackground(new Color(255, 255, 255));
    }
    
    public void paintComponent(Graphics g)
    {
        Graphics2D g2 = (Graphics2D) g;
        
        g2.setPaint(strokePaint);
        g2.setClip(0, 0, getWidth(), getHeight());
        
        g2.setStroke(stroke);
        
        Rectangle contentBounds = content.getBounds();
        Iterator<Target> targetIterator = targets.iterator();
        while (targetIterator.hasNext()) {
            Line2D line = targetIterator.next().connector;
            drawEllipses(line, contentBounds, g2);
        }
        
        double x, y, width, height;
        x = contentBounds.x - contentBounds.width  / 3;
        y = contentBounds.y - contentBounds.height / 3;
        width  = contentBounds.width  * 1.66;
        height = contentBounds.height * 1.66;
        g2.setPaint(background);
        g.fillOval((int)x, (int)y, (int)width, (int)height);
        g2.setPaint(strokePaint);
        g.drawOval((int)x, (int)y, (int)width, (int)height);
    }
    
    private void drawEllipses(Line2D line, Rectangle content, Graphics2D g)
    {
        double xSource, ySource, xTarget, yTarget;
        xSource = line.getX1();
        ySource = line.getY1();
        xTarget = line.getX2();
        yTarget = line.getY2();
        
        double contentRadius;
        contentRadius = Math.sqrt((content.width/2) * (content.width/2) + (content.height/2) * (content.height/2));
        
        double pathLength;
        pathLength = Math.sqrt((xTarget - xSource) * (xTarget - xSource) + (yTarget - ySource) * (yTarget - ySource)) - contentRadius;
        if (0 == pathLength) return;
        
        double shorteningFactor = pathLength / (pathLength + contentRadius);
        xSource = xTarget - (xTarget - xSource) * shorteningFactor;
        ySource = yTarget - (yTarget - ySource) * shorteningFactor;
        double width, height;
        double x, y;
        double deltaX = xTarget - xSource;
        double deltaY = yTarget - ySource;
        x = xSource;
        y = ySource;
        
        width = pathLength / 6;
        height = pathLength / 10;
        x = xSource + (deltaX / 3) * 2.8 - width  / 2;
        y = ySource + (deltaY / 3) * 2.8 - height / 2;
        g.setPaint(background);
        g.fillOval((int)x, (int)y, (int)width, (int)height);
        g.setPaint(strokePaint);
        g.drawOval((int)x, (int)y, (int)width, (int)height);
        
        width = pathLength / 4;
        height = pathLength / 6;
        x = xSource + (deltaX / 3) * 2.1 - width  / 2;
        y = ySource + (deltaY / 3) * 2.1 - height / 2;
        g.setPaint(background);
        g.fillOval((int)x, (int)y, (int)width, (int)height);
        g.setPaint(strokePaint);
        g.drawOval((int)x, (int)y, (int)width, (int)height);
        
        width = pathLength / 2;
        height = pathLength / 3;
        x = xSource + (deltaX / 3) * 1 - width  / 2;
        y = ySource + (deltaY / 3) * 1 - height / 2;
        g.setPaint(background);
        g.fillOval((int)x, (int)y, (int)width, (int)height);
        g.setPaint(strokePaint);
        g.drawOval((int)x, (int)y, (int)width, (int)height);
    }
    
}
